@@ -34,13 +34,13 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <gtk/gtk.h>
 #include <polkit/polkit.h>
 #include <libgnome-control-center/cc-panel.h>
+#include <timezonemap/cc-timezone-map.h>
+#include <timezonemap/timezone-completion.h>
 
 #include "dbus-shared.h"
 #include "settings-shared.h"
 #include "utils.h"
 #include "datetime-prefs-locations.h"
-#include "timezone-completion.h"
-#include "cc-timezone-map.h"
 
 #define DATETIME_DIALOG_UI_FILE PKGDATADIR "/datetime-dialog.ui"
 
@@ -69,7 +69,9 @@ struct _IndicatorDatetimePanelPrivate
   gboolean             user_edited_time;
   gboolean             changing_time;
   GtkWidget *          loc_dlg;
-  TimezoneCompletion * completion;
+  CcTimezoneCompletion * completion;
+  GCancellable         * tz_query_cancel;
+  GCancellable         * ntp_query_cancel;
 };
 
 struct _IndicatorDatetimePanelClass
@@ -214,6 +216,8 @@ ntp_query_answered (GObject *object, GAsyncResult *res, IndicatorDatetimePanel *
   GError * error = NULL;
   GVariant * answers = g_dbus_proxy_call_finish (G_DBUS_PROXY (object), res, &error);
 
+  g_clear_object (&self->priv->ntp_query_cancel);
+
   if (error != NULL) {
     g_warning("Could not query DBus proxy for SettingsDaemon: %s", error->message);
     g_error_free(error);
@@ -243,15 +247,20 @@ sync_entry (IndicatorDatetimePanel * self, const gchar * location)
 }
 
 static void
-tz_changed (CcTimezoneMap * map, TzLocation * location, IndicatorDatetimePanel * self)
+tz_changed (CcTimezoneMap * map, CcTimezoneLocation * location, IndicatorDatetimePanel * self)
 {
   if (location == NULL)
     return;
 
-  g_dbus_proxy_call (self->priv->proxy, "SetTimezone", g_variant_new ("(s)", location->zone),
+  gchar * zone;
+  g_object_get (location, "zone", &zone, NULL);
+
+  g_dbus_proxy_call (self->priv->proxy, "SetTimezone", g_variant_new ("(s)", zone),
                      G_DBUS_CALL_FLAGS_NONE, -1, NULL, dbus_set_answered, "timezone");
 
-  sync_entry (self, location->zone);
+  sync_entry (self, zone);
+
+  g_free (zone);
 }
 
 static void
@@ -260,6 +269,8 @@ tz_query_answered (GObject *object, GAsyncResult *res, IndicatorDatetimePanel *
   GError * error = NULL;
   GVariant * answers = g_dbus_proxy_call_finish (G_DBUS_PROXY (object), res, &error);
 
+  g_clear_object (&self->priv->tz_query_cancel);
+
   if (error != NULL) {
     g_warning("Could not query DBus proxy for SettingsDaemon: %s", error->message);
     g_error_free(error);
@@ -281,6 +292,7 @@ static void
 proxy_ready (GObject *object, GAsyncResult *res, IndicatorDatetimePanel * self)
 {
   GError * error = NULL;
+  IndicatorDatetimePanelPrivate * priv = self->priv;
 
   self->priv->proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
 
@@ -291,10 +303,16 @@ proxy_ready (GObject *object, GAsyncResult *res, IndicatorDatetimePanel * self)
   }
 
   /* And now, do initial proxy configuration */
-  g_dbus_proxy_call (self->priv->proxy, "GetUsingNtp", NULL, G_DBUS_CALL_FLAGS_NONE, -1,
-                     NULL, (GAsyncReadyCallback)ntp_query_answered, self);
-  g_dbus_proxy_call (self->priv->proxy, "GetTimezone", NULL, G_DBUS_CALL_FLAGS_NONE, -1,
-                     NULL, (GAsyncReadyCallback)tz_query_answered, self);
+  if (priv->ntp_query_cancel == NULL) {
+    priv->ntp_query_cancel = g_cancellable_new();
+    g_dbus_proxy_call (priv->proxy, "GetUsingNtp", NULL, G_DBUS_CALL_FLAGS_NONE, -1,
+                       priv->ntp_query_cancel, (GAsyncReadyCallback)ntp_query_answered, self);
+  }
+  if (priv->tz_query_cancel == NULL); {
+    priv->tz_query_cancel = g_cancellable_new();
+    g_dbus_proxy_call (priv->proxy, "GetTimezone", NULL, G_DBUS_CALL_FLAGS_NONE, -1,
+                       priv->tz_query_cancel, (GAsyncReadyCallback)tz_query_answered, self);
+  }
 }
 
 static void
@@ -400,6 +418,7 @@ input_time_text (GtkWidget * spinner, gdouble * value, IndicatorDatetimePanel *
     if (is_locale_12h ()) { // TODO: make this look-at/watch gsettings?
       char ampm[51];
 
+      /* coverity[secure_coding] */
       scanned = sscanf (text, "%u:%u:%u %50s", &hour_in, &minute_in, &second_in, ampm);
       passed = (scanned == 4);
 
@@ -410,6 +429,7 @@ input_time_text (GtkWidget * spinner, gdouble * value, IndicatorDatetimePanel *
         }
       }
     } else {
+      /* coverity[secure_coding] */
       scanned = sscanf (text, "%u:%u:%u", &hour_in, &minute_in, &second_in);
       passed = (scanned == 3);
     }
@@ -428,6 +448,7 @@ input_time_text (GtkWidget * spinner, gdouble * value, IndicatorDatetimePanel *
   else {
     gint year_in, month_in, day_in;
 
+    /* coverity[secure_coding] */
     scanned = sscanf (text, "%u-%u-%u", &year_in, &month_in, &day_in);
 
     if (scanned != 3 || year_in < 1 || year_in > 9999 ||
@@ -555,22 +576,15 @@ setup_time_spinners (IndicatorDatetimePanel * self, GtkWidget * time, GtkWidget
   update_spinners (self);
 }
 
-static void
-hide_locations (IndicatorDatetimePanel * self)
-{
-  if (self->priv->loc_dlg != NULL)
-    gtk_widget_destroy (self->priv->loc_dlg);
-}
-
 static void
 show_locations (IndicatorDatetimePanel * self)
 {
   if (self->priv->loc_dlg == NULL) {
     self->priv->loc_dlg = datetime_setup_locations_dialog (self->priv->tzmap);
     GtkWidget * dlg = gtk_widget_get_toplevel (GTK_WIDGET (self));
+    gtk_window_set_type_hint (GTK_WINDOW(self->priv->loc_dlg), GDK_WINDOW_TYPE_HINT_DIALOG);
     gtk_window_set_transient_for (GTK_WINDOW (self->priv->loc_dlg), GTK_WINDOW (dlg));
     g_signal_connect (self->priv->loc_dlg, "destroy", G_CALLBACK (gtk_widget_destroyed), &self->priv->loc_dlg);
-    g_signal_connect_swapped (dlg, "focus-in-event", G_CALLBACK (hide_locations), self);
     gtk_widget_show_all (self->priv->loc_dlg);
   }
   else {
@@ -585,8 +599,8 @@ timezone_selected (GtkEntryCompletion * widget, GtkTreeModel * model,
   const gchar * name, * zone;
 
   gtk_tree_model_get (model, iter,
-                      TIMEZONE_COMPLETION_NAME, &name,
-                      TIMEZONE_COMPLETION_ZONE, &zone,
+                      CC_TIMEZONE_COMPLETION_NAME, &name,
+                      CC_TIMEZONE_COMPLETION_ZONE, &zone,
                       -1);
 
   if (zone == NULL || zone[0] == 0) {
@@ -594,8 +608,8 @@ timezone_selected (GtkEntryCompletion * widget, GtkTreeModel * model,
     gdouble lon = 0.0, lat = 0.0;
 
     gtk_tree_model_get (model, iter,
-                        TIMEZONE_COMPLETION_LONGITUDE, &strlon,
-                        TIMEZONE_COMPLETION_LATITUDE, &strlat,
+                        CC_TIMEZONE_COMPLETION_LONGITUDE, &strlon,
+                        CC_TIMEZONE_COMPLETION_LATITUDE, &strlat,
                         -1);
 
     if (strlon != NULL && strlon[0] != 0) {
@@ -626,13 +640,17 @@ entry_focus_out (GtkEntry * entry, GdkEventFocus * event, IndicatorDatetimePanel
   // If the name left in the entry doesn't match the current timezone name,
   // show an error icon.  It's always an error for the user to manually type in
   // a timezone.
-  TzLocation * location = cc_timezone_map_get_location (self->priv->tzmap);
+  CcTimezoneLocation * location = cc_timezone_map_get_location (self->priv->tzmap);
   if (location == NULL)
     return FALSE;
 
-  gchar * name = get_current_zone_name (location->zone);
+  gchar * zone;
+  g_object_get (location, "zone", &zone, NULL);
+
+  gchar * name = get_current_zone_name (zone);
   gboolean correct = (g_strcmp0 (gtk_entry_get_text (entry), name) == 0);
   g_free (name);
+  g_free (zone);
 
   gtk_entry_set_icon_from_stock (entry, GTK_ENTRY_ICON_SECONDARY,
                                  correct ? NULL : GTK_STOCK_DIALOG_ERROR);
@@ -685,8 +703,8 @@ indicator_datetime_panel_init (IndicatorDatetimePanel * self)
   cc_timezone_map_set_watermark (self->priv->tzmap, "Geonames.org");
 
   /* And completion entry */
-  self->priv->completion = timezone_completion_new ();
-  timezone_completion_watch_entry (self->priv->completion, GTK_ENTRY (WIG ("timezoneEntry")));
+  self->priv->completion = cc_timezone_completion_new ();
+  cc_timezone_completion_watch_entry (self->priv->completion, GTK_ENTRY (WIG ("timezoneEntry")));
   g_signal_connect (self->priv->completion, "match-selected", G_CALLBACK (timezone_selected), self);
   g_signal_connect (WIG ("timezoneEntry"), "focus-out-event", G_CALLBACK (entry_focus_out), self);
 
@@ -767,46 +785,49 @@ static void
 indicator_datetime_panel_dispose (GObject * object)
 {
   IndicatorDatetimePanel * self = (IndicatorDatetimePanel *) object;
+  IndicatorDatetimePanelPrivate * priv = self->priv;
 
-  if (self->priv->builder) {
-    g_object_unref (self->priv->builder);
-    self->priv->builder = NULL;
+  g_clear_object (&priv->builder);
+  g_clear_object (&priv->proxy);
+
+  if (priv->tz_query_cancel != NULL) {
+    g_cancellable_cancel (priv->tz_query_cancel);
+    g_clear_object (&priv->tz_query_cancel);
   }
 
-  if (self->priv->proxy) {
-    g_object_unref (self->priv->proxy);
-    self->priv->proxy = NULL;
+  if (priv->ntp_query_cancel != NULL) {
+    g_cancellable_cancel (priv->ntp_query_cancel);
+    g_clear_object (&priv->ntp_query_cancel);
   }
 
-  if (self->priv->loc_dlg) {
-    gtk_widget_destroy (self->priv->loc_dlg);
-    self->priv->loc_dlg = NULL;
+  if (priv->loc_dlg) {
+    gtk_widget_destroy (priv->loc_dlg);
+    priv->loc_dlg = NULL;
   }
 
-  if (self->priv->save_time_id) {
-    g_source_remove (self->priv->save_time_id);
-    self->priv->save_time_id = 0;
+  if (priv->save_time_id) {
+    g_source_remove (priv->save_time_id);
+    priv->save_time_id = 0;
   }
 
-  if (self->priv->completion) {
-    timezone_completion_watch_entry (self->priv->completion, NULL);
-    g_object_unref (self->priv->completion);
-    self->priv->completion = NULL;
+  if (priv->completion) {
+    cc_timezone_completion_watch_entry (priv->completion, NULL);
+    g_clear_object (&priv->completion);
   }
 
-  if (self->priv->tz_entry) {
-    gtk_widget_destroy (self->priv->tz_entry);
-    self->priv->tz_entry = NULL;
+  if (priv->tz_entry) {
+    gtk_widget_destroy (priv->tz_entry);
+    priv->tz_entry = NULL;
   }
 
-  if (self->priv->time_spin) {
-    gtk_widget_destroy (self->priv->time_spin);
-    self->priv->time_spin = NULL;
+  if (priv->time_spin) {
+    gtk_widget_destroy (priv->time_spin);
+    priv->time_spin = NULL;
   }
 
-  if (self->priv->date_spin) {
-    gtk_widget_destroy (self->priv->date_spin);
-    self->priv->date_spin = NULL;
+  if (priv->date_spin) {
+    gtk_widget_destroy (priv->date_spin);
+    priv->date_spin = NULL;
   }
 }
 
