@@ -31,7 +31,7 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <math.h>
 #include <gconf/gconf-client.h>
 
-#include <libdbusmenu-gtk3/menuitem.h>
+#include <libdbusmenu-gtk/menuitem.h>
 #include <libdbusmenu-glib/server.h>
 #include <libdbusmenu-glib/client.h>
 #include <libdbusmenu-glib/menuitem.h>
@@ -54,15 +54,22 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "settings-shared.h"
 #include "utils.h"
 
+#ifdef HAVE_CCPANEL
+ #define SETTINGS_APP_INVOCATION "gnome-control-center indicator-datetime"
+#else
+ #define SETTINGS_APP_INVOCATION "gnome-control-center datetime"
+#endif
 
 static void geo_create_client (GeoclueMaster * master, GeoclueMasterClient * client, gchar * path, GError * error, gpointer user_data);
 static gboolean update_appointment_menu_items (gpointer user_data);
-static gboolean update_timezone_menu_items(gpointer user_data);
+static void update_location_menu_items (void);
 static void setup_timer (void);
 static void geo_client_invalid (GeoclueMasterClient * client, gpointer user_data);
 static void geo_address_change (GeoclueMasterClient * client, gchar * a, gchar * b, gchar * c, gchar * d, gpointer user_data);
 static gboolean get_greeter_mode (void);
 
+static void quick_set_tz (DbusmenuMenuitem * menuitem, guint timestamp, gpointer user_data);
+
 static IndicatorService * service = NULL;
 static GMainLoop * mainloop = NULL;
 static DbusmenuServer * server = NULL;
@@ -75,17 +82,14 @@ static DbusmenuMenuitem * calendar = NULL;
 static DbusmenuMenuitem * settings = NULL;
 static DbusmenuMenuitem * events_separator = NULL;
 static DbusmenuMenuitem * locations_separator = NULL;
-static DbusmenuMenuitem * geo_location = NULL;
-static DbusmenuMenuitem * current_location = NULL;
-//static DbusmenuMenuitem * ecal_location = NULL;
 static DbusmenuMenuitem * add_appointment = NULL;
-static GList			* appointments = NULL;
-static GList			* dconflocations = NULL;
-static GList			* comp_instances = NULL;
+static GList            * appointments = NULL;
+static GSList           * location_menu_items = NULL;
+static GList            * comp_instances = NULL;
 static gboolean           updating_appointments = FALSE;
-static time_t			  start_time_appointments = (time_t) 0;
-GSettings *conf;
-GConfClient* gconf;
+static time_t             start_time_appointments = (time_t) 0;
+static GSettings        * conf = NULL;
+static GConfClient      * gconf = NULL;
 
 
 /* Geoclue trackers */
@@ -103,115 +107,144 @@ struct comp_instance {
         ESource *source;
 };
 
-static void
-set_timezone_label (DbusmenuMenuitem * mi, const gchar * location)
+/**
+ * A temp struct used by update_location_menu_items() for pruning duplicates and sorting.
+ */
+struct TimeLocation
 {
-	gchar * zone, * name;
-	split_settings_location (location, &zone, &name);
-
-	dbusmenu_menuitem_property_set (mi, TIMEZONE_MENUITEM_PROP_NAME, name);
-	dbusmenu_menuitem_property_set (mi, TIMEZONE_MENUITEM_PROP_ZONE, zone);
-
-	g_free (zone);
-	g_free (name);
-}
-
+	gint32 offset;
+	gchar * zone;
+	gchar * name;
+};
 static void
-set_current_timezone_label (DbusmenuMenuitem * mi, const gchar * location)
+time_location_free (struct TimeLocation * loc)
 {
-	gchar * name = get_current_zone_name (location);
-
-	dbusmenu_menuitem_property_set (mi, TIMEZONE_MENUITEM_PROP_NAME, name);
-	dbusmenu_menuitem_property_set (mi, TIMEZONE_MENUITEM_PROP_ZONE, location);
-
-	g_free (name);
+	g_free (loc->name);
+	g_free (loc->zone);
+	g_free (loc);
 }
+static struct TimeLocation*
+time_location_new (const char * zone, const char * name, time_t now)
+{
+	struct TimeLocation * loc = g_new (struct TimeLocation, 1);
+	GTimeZone * tz = g_time_zone_new (zone);
+	gint interval = g_time_zone_find_interval (tz, G_TIME_TYPE_UNIVERSAL, now);
+	loc->offset = g_time_zone_get_offset (tz, interval);
+	loc->zone = g_strdup (zone);
+	loc->name = g_strdup (name);
+	g_time_zone_unref (tz);
+	g_debug ("%s zone '%s' name '%s' offset is %d", G_STRLOC, zone, name, (int)loc->offset);
+	return loc;
+}
+static int
+time_location_compare (const struct TimeLocation * a, const struct TimeLocation * b)
+{
+	int ret;
+	if (a->offset != b->offset) /* primary key s.t. we can sort by timezone order */
+		ret = a->offset - b->offset;
+	else
+		ret = g_strcmp0 (a->name, b->name); /* secondary key */
+	g_debug ("%s comparing '%s' (%d) to '%s' (%d), returning %d", G_STRLOC, a->name, (int)a->offset, b->name, (int)b->offset, ret);
+	return ret;
+}
+static GSList*
+locations_add (GSList * locations, const char * zone, const char * name, time_t now)
+{
+	struct TimeLocation * loc = time_location_new (zone, name, now);
 
-/* Check to see if our timezones are the same */
-static void
-check_timezone_sync (void) {
-	gchar * label;
-	gboolean in_sync = FALSE;
-	
-	if (geo_timezone == NULL) {
-		in_sync = TRUE;
-	}
-
-	if (current_timezone == NULL) {
-		in_sync = TRUE;
-	}
-
-	if (!in_sync && g_strcmp0(geo_timezone, current_timezone) == 0) {
-		in_sync = TRUE;
-	}
-
-	if (in_sync) {
-		g_debug("Timezones in sync");
+	if (g_slist_find_custom (locations, loc, (GCompareFunc)time_location_compare) == NULL) {
+		g_debug ("%s Adding zone '%s', name '%s'", G_STRLOC, zone, name);
+		locations = g_slist_prepend (locations, loc);
 	} else {
-		g_debug("Timezones are different");
+		g_debug("%s Skipping duplicate zone '%s' name '%s'", G_STRLOC, zone, name);
+		time_location_free (loc);
+	}
+	return locations;
+}
+
+/* Update the timezone entries */
+static void
+update_location_menu_items (void)
+{
+	/* if we're in greeter mode, don't bother */
+	if (locations_separator == NULL)
+		return;
+
+	/* remove the previous locations */
+	while (location_menu_items != NULL) {
+		DbusmenuMenuitem * item = DBUSMENU_MENUITEM(location_menu_items->data);
+		location_menu_items = g_slist_remove(location_menu_items, item);
+		dbusmenu_menuitem_child_delete(root, DBUSMENU_MENUITEM(item));
+		g_object_unref(G_OBJECT(item));
 	}
 
-	gboolean show = g_settings_get_boolean (conf, SETTINGS_SHOW_LOCATIONS_S);
+	/***
+	****  Build a list of locations to add: use geo_timezone,
+	****  current_timezone, and SETTINGS_LOCATIONS_S, but omit duplicates.
+	***/
 
-	if (geo_location != NULL && current_location != NULL) {
-		g_debug("Got timezone %s", current_timezone);
-		g_debug("Got timezone %s", geo_timezone);
-		// Show neither current location nor geo location if both are the same
-		// however, we want to set their time and label accordingly
-		if (in_sync) {
-			if (current_timezone == NULL && geo_timezone == NULL) {
-				dbusmenu_menuitem_property_set_bool(locations_separator, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
-				dbusmenu_menuitem_property_set_bool (current_location, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
-				dbusmenu_menuitem_property_set_bool (geo_location, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
-				update_timezone_menu_items(NULL); // Update the timezone menu items 
-				return;
-			}
-			
-			dbusmenu_menuitem_property_set_bool (locations_separator, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
-			dbusmenu_menuitem_property_set_bool (current_location, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
-			dbusmenu_menuitem_property_set_bool (current_location, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
-			dbusmenu_menuitem_property_set_bool (geo_location, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
-			dbusmenu_menuitem_property_set_bool (geo_location, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
-			
-			if (current_timezone != NULL) {
-				label = current_timezone;
-			} else {
-				label = geo_timezone;
-			}
-			
-			if (label != NULL) {
-				// TODO work out the current location name in a nice way
-				set_current_timezone_label (current_location, label);
-				// TODO work out the current time at that location 
-				dbusmenu_menuitem_property_set_bool (current_location, DBUSMENU_MENUITEM_PROP_VISIBLE, show);
-				dbusmenu_menuitem_property_set_bool(current_location, TIMEZONE_MENUITEM_PROP_RADIO, TRUE);
-			} else {
-				g_debug("Label for current location is null, this shouldn't happen");
-			}
-			if (geo_timezone != NULL) {	
-				// TODO work out the geo location name in a nice way
-				set_current_timezone_label (geo_location, geo_timezone);
-				// TODO work out the current time at that location 
-				dbusmenu_menuitem_property_set_bool (geo_location, DBUSMENU_MENUITEM_PROP_VISIBLE, show);
-			}
-		} else {
-			// TODO work out the geo location name in a nice way
-			set_current_timezone_label (geo_location, geo_timezone);
-			// TODO work out the current time at that location 
-			dbusmenu_menuitem_property_set_bool(geo_location, DBUSMENU_MENUITEM_PROP_VISIBLE, show);
-			
-			// TODO work out the current location name in a nice way
-			set_current_timezone_label (current_location, current_timezone);
-			// TODO work out the current time at that location 
-			dbusmenu_menuitem_property_set_bool(current_location, TIMEZONE_MENUITEM_PROP_RADIO, TRUE);
-			dbusmenu_menuitem_property_set_bool(current_location, DBUSMENU_MENUITEM_PROP_VISIBLE, show);
-			dbusmenu_menuitem_property_set_bool(locations_separator, DBUSMENU_MENUITEM_PROP_VISIBLE, show);
+	GSList * locations = NULL;
+	const time_t now = time(NULL);
+
+	/* maybe add geo_timezone */
+	if (geo_timezone != NULL) {
+		gchar * name = get_current_zone_name (geo_timezone);
+		locations = locations_add (locations, geo_timezone, name, now);
+		g_free (name);
+	}
+
+	/* maybe add current_timezone */
+	if (current_timezone != NULL) {
+		gchar * name = get_current_zone_name (current_timezone);
+		locations = locations_add (locations, current_timezone, name, now);
+		g_free (name);
+	}
+
+	/* maybe add the user-specified custom locations */
+	gchar ** user_locations = g_settings_get_strv (conf, SETTINGS_LOCATIONS_S);
+	if (user_locations != NULL) { 
+		gint i;
+		const guint location_count = g_strv_length (user_locations);
+		g_debug ("%s Found %u user-specified locations", G_STRLOC, location_count);
+		for (i=0; i<location_count; i++) {
+			gchar * zone;
+			gchar * name;
+			split_settings_location (user_locations[i], &zone, &name);
+			locations = locations_add (locations, zone, name, now);
+			g_free (name);
+			g_free (zone);
 		}
+		g_strfreev (user_locations);
+		user_locations = NULL;
 	}
-	g_debug("Finished checking timezone sync");
-	update_timezone_menu_items(NULL); // Update the timezone menu items 
 
-	return;
+	/* sort the list by timezone offset */
+	locations = g_slist_sort (locations, (GCompareFunc)time_location_compare);
+
+	/* finally create menuitems for each location */
+	gint offset = dbusmenu_menuitem_get_position (locations_separator, root)+1;
+	const gboolean show_locations = g_settings_get_boolean (conf, SETTINGS_SHOW_LOCATIONS_S);
+	GSList * l;
+	for (l=locations; l!=NULL; l=l->next) {
+		struct TimeLocation * loc = l->data;
+		g_debug("%s Adding location: zone '%s', name '%s'", G_STRLOC, loc->zone, loc->name);
+		DbusmenuMenuitem * item = dbusmenu_menuitem_new();
+		dbusmenu_menuitem_property_set      (item, DBUSMENU_MENUITEM_PROP_TYPE, TIMEZONE_MENUITEM_TYPE);
+		dbusmenu_menuitem_property_set      (item, TIMEZONE_MENUITEM_PROP_NAME, loc->name);
+		dbusmenu_menuitem_property_set      (item, TIMEZONE_MENUITEM_PROP_ZONE, loc->zone);
+		dbusmenu_menuitem_property_set_bool (item, TIMEZONE_MENUITEM_PROP_RADIO, FALSE);
+		dbusmenu_menuitem_property_set_bool (item, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
+		dbusmenu_menuitem_property_set_bool (item, DBUSMENU_MENUITEM_PROP_VISIBLE, show_locations);
+		dbusmenu_menuitem_child_add_position (root, item, offset++);
+		g_signal_connect(G_OBJECT(item), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED, G_CALLBACK(quick_set_tz), NULL);
+		location_menu_items = g_slist_append (location_menu_items, item);
+		time_location_free (loc);
+	}
+	g_slist_free (locations);
+	locations = NULL;
+
+	/* if there's at least one item being shown, show the separator too */
+	dbusmenu_menuitem_property_set_bool (locations_separator, DBUSMENU_MENUITEM_PROP_VISIBLE, show_locations && (location_menu_items!=NULL));
 }
 
 /* Update the current timezone */
@@ -230,7 +263,7 @@ update_current_timezone (void) {
 
 	g_debug("System timezone is: %s", current_timezone);
 
-	check_timezone_sync();
+	update_location_menu_items();
 
 	return;
 }
@@ -243,7 +276,7 @@ quick_set_tz_cb (GObject *object, GAsyncResult *res, gpointer data)
 
   if (error != NULL) {
     g_warning("Could not set timezone for SettingsDaemon: %s", error->message);
-    g_error_free(error);
+    g_clear_error (&error);
     return;
   }
 
@@ -259,7 +292,7 @@ quick_set_tz_proxy_cb (GObject *object, GAsyncResult *res, gpointer zone)
 
 	if (error != NULL) {
 		g_warning("Could not grab DBus proxy for SettingsDaemon: %s", error->message);
-		g_error_free(error);
+		g_clear_error (&error);
 		g_free (zone);
 		return;
 	}
@@ -326,17 +359,26 @@ update_datetime (gpointer user_data)
 
 /* Run a particular program based on an activation */
 static void
-activate_cb (DbusmenuMenuitem * menuitem, guint timestamp, const gchar *command)
+execute_command (const gchar * command)
 {
 	GError * error = NULL;
 
 	g_debug("Issuing command '%s'", command);
 	if (!g_spawn_command_line_async(command, &error)) {
 		g_warning("Unable to start %s: %s", (char *)command, error->message);
-		g_error_free(error);
+		g_clear_error (&error);
 	}
 }
 
+/* Run a particular program based on an activation */
+static void
+activate_cb (DbusmenuMenuitem  * menuitem  G_GNUC_UNUSED,
+             guint               timestamp G_GNUC_UNUSED,
+             const gchar       * command)
+{
+	execute_command (command);
+}
+
 static gboolean
 update_appointment_menu_items_idle (gpointer user_data)
 {
@@ -406,22 +448,22 @@ day_selected_cb (DbusmenuMenuitem * menuitem, gchar *name, GVariant *variant, gu
 }
 
 static gboolean
-day_selected_double_click_cb (DbusmenuMenuitem * menuitem, gchar *name, GVariant *variant, guint timestamp)
+day_selected_double_click_cb (DbusmenuMenuitem * menuitem  G_GNUC_UNUSED,
+                              gchar            * name      G_GNUC_UNUSED,
+                              GVariant         * variant,
+                              guint              timestamp G_GNUC_UNUSED)
 {
-	time_t evotime = (time_t)g_variant_get_uint32(variant);
+	const time_t evotime = (time_t)g_variant_get_uint32(variant);
 	
 	g_debug("Received day-selected-double-click with timestamp: %d -> %s",(int)evotime, ctime(&evotime));	
 	
 	gchar *ad = isodate_from_time_t(evotime);
 	gchar *cmd = g_strconcat("evolution calendar:///?startdate=", ad, NULL);
 	
-	GError * error = NULL;
+	execute_command (cmd);
 
-	g_debug("Issuing command '%s'", cmd);
-	if (!g_spawn_command_line_async(cmd, &error)) {
-		g_warning("Unable to start %s: %s", (char *)cmd, error->message);
-		g_error_free(error);
-	}
+	g_free (cmd);
+	g_free (ad);
 	
 	return TRUE;
 }
@@ -479,6 +521,24 @@ show_events_changed (void)
 	}
 }
 
+static gboolean
+calendar_app_is_usable (void)
+{
+	/* confirm that it's installed... */
+	gchar *evo = g_find_program_in_path("evolution");
+	if (evo == NULL)
+		return FALSE;
+	g_debug ("found calendar app: '%s'", evo);
+	g_free (evo);
+
+	/* confirm that it's got an account set up... */
+	GSList *accounts_list = gconf_client_get_list (gconf, "/apps/evolution/mail/accounts", GCONF_VALUE_STRING, NULL);
+	const guint n = g_slist_length (accounts_list);
+	g_debug ("found %u evolution accounts", n);
+	g_slist_free_full (accounts_list, g_free);
+	return n > 0;
+}
+
 /* Looks for the calendar application and enables the item if
    we have one, starts ecal timer if events are turned on */
 static gboolean
@@ -488,9 +548,7 @@ check_for_calendar (gpointer user_data)
 	
 	dbusmenu_menuitem_property_set_bool(date, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
 	
-	gchar *evo = g_find_program_in_path("evolution");
-	if (!get_greeter_mode () && evo != NULL) {
-		g_debug("Found the calendar application: %s", evo);
+	if (!get_greeter_mode () && calendar_app_is_usable()) {
 		
 		g_signal_connect (G_OBJECT(date), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED,
 		                  G_CALLBACK (activate_cb), "evolution -c calendar");
@@ -519,7 +577,6 @@ check_for_calendar (gpointer user_data)
 		g_signal_connect(calendar, "event::month-changed", G_CALLBACK(month_changed_cb), NULL);
 		g_signal_connect(calendar, "event::day-selected", G_CALLBACK(day_selected_cb), NULL);
 		g_signal_connect(calendar, "event::day-selected-double-click", G_CALLBACK(day_selected_double_click_cb), NULL);
-		g_free(evo);
 	} else {
 		g_debug("Unable to find calendar app.");
 		dbusmenu_menuitem_property_set_bool(add_appointment, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
@@ -537,68 +594,6 @@ check_for_calendar (gpointer user_data)
 	return FALSE;
 }
 
-
-static gboolean
-update_timezone_menu_items(gpointer user_data) {
-	g_debug("Updating timezone menu items");
-
-	if (locations_separator == NULL || current_location == NULL) {
-		return FALSE;
-	}
-
-	gchar ** locations = g_settings_get_strv(conf, SETTINGS_LOCATIONS_S);
-
-	if (locations == NULL) { 
-		g_debug("No locations configured (NULL)");
-		return FALSE;
-	} 
-	guint len = g_strv_length(locations);
-	DbusmenuMenuitem *item;
-	gint i, offset;
-	
-	/* Remove all of the previous locations */
-	if (dconflocations != NULL) {
-		while (dconflocations != NULL) {
-			DbusmenuMenuitem * litem =  DBUSMENU_MENUITEM(dconflocations->data);
-			// Remove all the existing menu items which are in dconflocations.
-			dconflocations = g_list_remove(dconflocations, litem);
-			dbusmenu_menuitem_child_delete(root, DBUSMENU_MENUITEM(litem));
-			g_object_unref(G_OBJECT(litem));
-		}
-	}
-	
-	gboolean show = g_settings_get_boolean (conf, SETTINGS_SHOW_LOCATIONS_S);
-
-	dbusmenu_menuitem_property_set_bool (locations_separator, DBUSMENU_MENUITEM_PROP_VISIBLE, show);
-	dbusmenu_menuitem_property_set_bool (current_location, DBUSMENU_MENUITEM_PROP_VISIBLE, show);
-	dbusmenu_menuitem_property_set_bool (current_location, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
-
-	if (len == 0) {
-		g_debug("No locations configured (Empty List)");
-		return FALSE;
-	}
-	
-	offset = dbusmenu_menuitem_get_position (current_location, root)+1;
-	for (i = 0; i < len; i++) {
-		// Iterate over configured places and add any which aren't already listed
-		if ((current_timezone == NULL || !g_str_has_prefix(locations[i], current_timezone)) &&
-		    (geo_timezone == NULL || !g_str_has_prefix(locations[i], geo_timezone))) {
-			g_debug("Adding timezone in update_timezones %s", locations[i]);
-			item = dbusmenu_menuitem_new();
-			dbusmenu_menuitem_property_set      (item, DBUSMENU_MENUITEM_PROP_TYPE, TIMEZONE_MENUITEM_TYPE);
-			set_timezone_label (item, locations[i]);
-			dbusmenu_menuitem_property_set_bool (item, TIMEZONE_MENUITEM_PROP_RADIO, FALSE);
-			dbusmenu_menuitem_property_set_bool (item, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
-			dbusmenu_menuitem_property_set_bool (item, DBUSMENU_MENUITEM_PROP_VISIBLE, show);
-			dbusmenu_menuitem_child_add_position (root, item, offset++);
-			g_signal_connect(G_OBJECT(item), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED, G_CALLBACK(quick_set_tz), NULL);
-			dconflocations = g_list_append(dconflocations, item);
-		}
-	}
-	g_strfreev (locations);
-	return FALSE;
-}
-
 // Authentication function
 static gchar *
 auth_func (ECal *ecal, 
@@ -621,22 +616,43 @@ auth_func (ECal *ecal,
 }
 
 static gint
-compare_comp_instances (gconstpointer a, 
-                        gconstpointer b)
+compare_comp_instances (gconstpointer ga, gconstpointer gb)
 {
-        const struct comp_instance *ci_a = a;
-        const struct comp_instance *ci_b = b;
-        time_t d = ci_a->start - ci_b->start;
-		if (d < 0) return -1;
-		else if (d > 0) return 1; 
-		return 0;
+	const struct comp_instance * a = ga;
+	const struct comp_instance * b = gb;
+
+	/* sort by start time */
+	if (a->start < b->start) return -1;
+	if (a->start > b->start) return  1;
+	return 0;
+}
+
+static struct comp_instance*
+comp_instance_new (ECalComponent * comp, time_t start, time_t end, ESource * source)
+{
+	g_debug("Using times start %s, end %s", ctime(&start), ctime(&end));
+
+	struct comp_instance *ci = g_new (struct comp_instance, 1);
+	ci->comp = g_object_ref (comp);
+	ci->source = source;
+	ci->start = start;
+	ci->end = end;
+	return ci;
+}
+static void
+comp_instance_free (struct comp_instance* ci)
+{
+	if (ci != NULL) {
+		g_clear_object (&ci->comp);
+		g_free (ci);
+	}
 }
 
 static gboolean
-populate_appointment_instances (ECalComponent *comp,
-                                time_t instance_start,
-                                time_t instance_end,
-                                gpointer data)
+populate_appointment_instances (ECalComponent * comp,
+                                time_t          start,
+                                time_t          end,
+                                gpointer        data)
 {
 	g_debug("Appending item %p", comp);
 	
@@ -646,38 +662,9 @@ populate_appointment_instances (ECalComponent *comp,
 	icalproperty_status status;
 	e_cal_component_get_status (comp, &status);
 	if (status == ICAL_STATUS_COMPLETED || status == ICAL_STATUS_CANCELLED) return FALSE;
-	
-	g_object_ref(comp);
-	
-	ECalComponentDateTime datetime;
-	icaltimezone *appointment_zone = NULL;
-	icaltimezone *current_zone = NULL;
-	
-	if (vtype == E_CAL_COMPONENT_EVENT)
-		e_cal_component_get_dtstart (comp, &datetime);
-	else
-	    e_cal_component_get_due (comp, &datetime);
 
-	appointment_zone = icaltimezone_get_builtin_timezone_from_tzid(datetime.tzid);
-	current_zone = icaltimezone_get_builtin_timezone_from_tzid(current_timezone);
-	if (!appointment_zone || datetime.value->is_date) { // If it's today put in the current timezone?
-		appointment_zone = current_zone;
-	}
-	
-	// TODO: Convert the timezone into a 3 letter abbreviation if it's different to current_timezone
-	// TODO: Add the appointment timezone to the list if it's not already there. 
-	
-	struct comp_instance *ci;
-	ci = g_new (struct comp_instance, 1);
-	
-	g_debug("Using times start %s, end %s", ctime(&instance_start), ctime(&instance_end));
-	
-	ci->comp = comp;
-	ci->source = E_SOURCE(data);
-	ci->start = instance_start;
-	ci->end = instance_end;
-	
-	comp_instances = g_list_append(comp_instances, ci);
+	struct comp_instance *ci = comp_instance_new (comp, start, end, E_SOURCE(data));
+	comp_instances = g_list_append (comp_instances, ci);
 	return TRUE;
 }
 
@@ -698,7 +685,6 @@ update_appointment_menu_items (gpointer user_data)
 	updating_appointments = TRUE;
 	
 	time_t curtime = 0, t1 = 0, t2 = 0;
-	gchar *ad;
 	GList *l;
 	GSList *g;
 	GError *gerror = NULL;
@@ -746,25 +732,18 @@ update_appointment_menu_items (gpointer user_data)
 	
 	if (!e_cal_get_sources(&sources, E_CAL_SOURCE_TYPE_EVENT, &gerror)) {
 		g_debug("Failed to get ecal sources\n");
+		g_clear_error (&gerror);
 		return FALSE;
 	}
 	
-	// Free comp_instances if not NULL
-	if (comp_instances != NULL) {
-		g_debug("Freeing comp_instances: may be an overlap\n");
-		for (l = comp_instances; l; l = l->next) {
-			const struct comp_instance *ci = l->data;
-			g_object_unref(ci->comp);
-		}
-		g_list_free(comp_instances);
-		comp_instances = NULL;
+	// clear any previous comp_instances
+	g_list_free_full (comp_instances, (GDestroyNotify)comp_instance_free);
+	comp_instances = NULL;
 
-	}
 	GSList *cal_list = gconf_client_get_list(gconf, "/apps/evolution/calendar/display/selected_calendars", GCONF_VALUE_STRING, &gerror);
 	if (gerror) {
 	  g_debug("Failed to get evolution preference for enabled calendars");
-	  g_error_free(gerror);
-	  gerror = NULL;
+	  g_clear_error (&gerror);
 	  cal_list = NULL;
 	}
 	
@@ -786,39 +765,33 @@ update_appointment_menu_items (gpointer user_data)
 			}
 			if (current_zone && !e_cal_set_default_timezone(ecal, current_zone, &gerror)) {
 				g_debug("Failed to set ecal default timezone %s", gerror->message);
-				g_error_free(gerror);
-				gerror = NULL;
+				g_clear_error (&gerror);
 				g_object_unref(ecal);
 				continue;
 			}
 			
 			if (!e_cal_open(ecal, FALSE, &gerror)) {
 				g_debug("Failed to get ecal sources %s", gerror->message);
-				g_error_free(gerror);
-				gerror = NULL;
+				g_clear_error (&gerror);
 				g_object_unref(ecal);
 				continue;
 			}
+
 			const gchar *ecal_uid = e_source_peek_uid(source);
-			gboolean match = FALSE;
 			g_debug("Checking ecal_uid is enabled: %s", ecal_uid);
-			for (i = 0; i<g_slist_length(cal_list);i++) {
-				char *cuid = (char *)g_slist_nth_data(cal_list, i);
-				if (g_strcmp0(cuid, ecal_uid) == 0) {
-					match = TRUE;
-					break;
-				}
-			}
-			if (!match) {
+			const gboolean in_list = g_slist_find_custom (cal_list, ecal_uid, (GCompareFunc)g_strcmp0) != NULL;
+			if (!in_list) {
 				g_object_unref(ecal);
 				continue;
 			}
+
 			g_debug("ecal_uid is enabled, generating instances");
-			
-			e_cal_generate_instances (ecal, t1, t2, (ECalRecurInstanceFn) populate_appointment_instances, (gpointer) source);
+			e_cal_generate_instances (ecal, t1, t2, (ECalRecurInstanceFn) populate_appointment_instances, source);
 			g_object_unref(ecal);
 		}
 	}
+	g_slist_free_full (cal_list, g_free);
+
 	g_debug("Number of ECalComponents returned: %d", g_list_length(comp_instances));
 	GList *sorted_comp_instances = g_list_sort(comp_instances, compare_comp_instances);
 	comp_instances = NULL;
@@ -848,24 +821,21 @@ update_appointment_menu_items (gpointer user_data)
 		apt_output = SETTINGS_TIME_12_HOUR;
 	} else if (g_strcmp0(time_format_str, "24-hour") == 0) {
 		apt_output = SETTINGS_TIME_24_HOUR;
+	} else if (is_locale_12h()) {
+		apt_output = SETTINGS_TIME_12_HOUR;
 	} else {
-		if (is_locale_12h()) {
-			apt_output = SETTINGS_TIME_12_HOUR;
-		} else {
-			apt_output = SETTINGS_TIME_24_HOUR;
-		}
+		apt_output = SETTINGS_TIME_24_HOUR;
 	}
+	g_free (time_format_str);
 	
 	GVariantBuilder markeddays;
-	g_variant_builder_init (&markeddays, G_VARIANT_TYPE_ARRAY);
+	g_variant_builder_init (&markeddays, G_VARIANT_TYPE ("ai"));
 	
 	i = 0;
 	GList * cached_appointment = appointments;
 	for (l = sorted_comp_instances; l; l = l->next) {
 		struct comp_instance *ci = l->data;
 		ECalComponent *ecalcomp = ci->comp;
-		ECalComponentText valuetext;
-		gchar *summary, *cmd;
 		char right[20];
 		//const gchar *uri;
 		DbusmenuMenuitem * item;
@@ -922,12 +892,11 @@ update_appointment_menu_items (gpointer user_data)
 
 	
         // Label text        
+		ECalComponentText valuetext;
 		e_cal_component_get_summary (ecalcomp, &valuetext);
-		summary = g_strdup (valuetext.value);
-
-		dbusmenu_menuitem_property_set (item, APPOINTMENT_MENUITEM_PROP_LABEL, summary);
+		const gchar * summary = valuetext.value;
 		g_debug("Summary: %s", summary);
-		g_free (summary);
+		dbusmenu_menuitem_property_set (item, APPOINTMENT_MENUITEM_PROP_LABEL, summary);
 
 		gboolean full_day = FALSE;
 		if (vtype == E_CAL_COMPONENT_EVENT) {
@@ -966,12 +935,12 @@ update_appointment_menu_items (gpointer user_data)
 		// Now we pull out the URI for the calendar event and try to create a URI that'll work when we execute evolution
 		// FIXME Because the URI stuff is really broken, we're going to open the calendar at todays date instead
 		//e_cal_component_get_uid(ecalcomp, &uri);
-		ad = isodate_from_time_t(mktime(due));
-		cmd = g_strconcat("evolution calendar:///?startdate=", ad, NULL);
-		g_signal_connect (G_OBJECT(item), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED,
-						  G_CALLBACK (activate_cb), cmd);
-		
+		gchar * ad = isodate_from_time_t(mktime(due));
+		gchar * cmd = g_strconcat("evolution calendar:///?startdate=", ad, NULL);
 		g_debug("Command to Execute: %s", cmd);
+		g_signal_connect_data (G_OBJECT(item), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED,
+		                       G_CALLBACK(activate_cb), cmd, (GClosureNotify)g_free, 0);
+		g_free (ad);
 
         const gchar *color_spec = e_source_peek_color_spec(ci->source);
         g_debug("Colour to use: %s", color_spec);
@@ -979,13 +948,19 @@ update_appointment_menu_items (gpointer user_data)
 		// Draw the correct icon for the appointment type and then tint it using mask fill.
 		// For now we'll create a circle
         if (color_spec != NULL) {
-        	GdkColor color;
-        	gdk_color_parse (color_spec, &color);	
         	g_debug("Creating a cairo surface: size, %d by %d", width, height);         
         	cairo_surface_t *surface = cairo_image_surface_create( CAIRO_FORMAT_ARGB32, width, height ); 
 
-    		cairo_t *cr = cairo_create(surface);
-			gdk_cairo_set_source_color(cr, &color);
+			cairo_t *cr = cairo_create(surface);
+#if GTK_CHECK_VERSION(3,0,0)
+        	GdkRGBA rgba;
+        	if (gdk_rgba_parse (&rgba, color_spec))
+        		gdk_cairo_set_source_rgba (cr, &rgba);
+#else
+			GdkColor color;
+			if (gdk_color_parse (color_spec, &color))
+				gdk_cairo_set_source_color (cr, &color);
+#endif
 			cairo_paint(cr);
     		cairo_set_source_rgba(cr, 0,0,0,0.5);
     		cairo_set_line_width(cr, 1);
@@ -1025,6 +1000,7 @@ update_appointment_menu_items (gpointer user_data)
 	  			}
 	  			
 				dbusmenu_menuitem_property_set_image (item, APPOINTMENT_MENUITEM_PROP_ICON, pixbuf);
+				g_clear_object (&pixbuf);
 			} else {
 				g_debug("Creating pixbuf from surface failed");
 			}
@@ -1034,15 +1010,15 @@ update_appointment_menu_items (gpointer user_data)
 		g_debug("Adding appointment: %p", item);
 	}
 	
-    if (gerror != NULL) g_error_free(gerror);
-	for (l = sorted_comp_instances; l; l = l->next) { 
-		const struct comp_instance *ci = l->data;
-		g_object_unref(ci->comp);
-	}
-	g_list_free(sorted_comp_instances);
+	g_clear_error (&gerror);
+
+	g_list_free_full (sorted_comp_instances, (GDestroyNotify)comp_instance_free);
+	sorted_comp_instances = NULL;
 	
 	GVariant * marks = g_variant_builder_end (&markeddays);
 	dbusmenu_menuitem_property_set_variant (calendar, CALENDAR_MENUITEM_PROP_MARKS, marks);
+
+	g_clear_object (&sources);
 	
 	updating_appointments = FALSE;
 	g_debug("End of objects");
@@ -1073,7 +1049,7 @@ static void
 show_locations_changed (void)
 {
 	/* Re-calculate */
-	check_timezone_sync();
+	update_location_menu_items();
 }
 
 static void
@@ -1116,22 +1092,7 @@ build_menus (DbusmenuMenuitem * root)
 		dbusmenu_menuitem_property_set_bool (locations_separator, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
 		dbusmenu_menuitem_child_append(root, locations_separator);
 
-		geo_location = dbusmenu_menuitem_new();
-		dbusmenu_menuitem_property_set      (geo_location, DBUSMENU_MENUITEM_PROP_TYPE, TIMEZONE_MENUITEM_TYPE);
-		set_current_timezone_label (geo_location, "");
-		dbusmenu_menuitem_property_set_bool (geo_location, DBUSMENU_MENUITEM_PROP_ENABLED, FALSE);
-		dbusmenu_menuitem_property_set_bool (geo_location, DBUSMENU_MENUITEM_PROP_VISIBLE, TRUE);
-		g_signal_connect(G_OBJECT(geo_location), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED, G_CALLBACK(quick_set_tz), NULL);
-		dbusmenu_menuitem_child_append(root, geo_location);
-
-		current_location = dbusmenu_menuitem_new();
-		dbusmenu_menuitem_property_set      (current_location, DBUSMENU_MENUITEM_PROP_TYPE, TIMEZONE_MENUITEM_TYPE);
-		set_current_timezone_label (current_location, "");
-		dbusmenu_menuitem_property_set_bool (current_location, DBUSMENU_MENUITEM_PROP_ENABLED, FALSE);
-		dbusmenu_menuitem_property_set_bool (current_location, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
-		dbusmenu_menuitem_child_append(root, current_location);
-	
-		check_timezone_sync();
+		update_location_menu_items();
 	
 		g_signal_connect (conf, "changed::" SETTINGS_SHOW_LOCATIONS_S, G_CALLBACK (show_locations_changed), NULL);
 		g_signal_connect (conf, "changed::" SETTINGS_LOCATIONS_S, G_CALLBACK (show_locations_changed), NULL);
@@ -1146,7 +1107,7 @@ build_menus (DbusmenuMenuitem * root)
 		dbusmenu_menuitem_property_set     (settings, DBUSMENU_MENUITEM_PROP_LABEL, _("Time & Date Settings…"));
 		/* insensitive until we check for available apps */
 		dbusmenu_menuitem_property_set_bool(settings, DBUSMENU_MENUITEM_PROP_ENABLED, FALSE);
-		g_signal_connect(G_OBJECT(settings), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED, G_CALLBACK(activate_cb), "gnome-control-center indicator-datetime");
+		g_signal_connect(G_OBJECT(settings), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED, G_CALLBACK(activate_cb), SETTINGS_APP_INVOCATION);
 		dbusmenu_menuitem_child_append(root, settings);
 		g_idle_add(check_for_timeadmin, NULL);
 	}
@@ -1244,7 +1205,7 @@ system_proxy_cb (GObject * object, GAsyncResult * res, gpointer user_data)
 
 	if (error != NULL) {
 		g_warning("Could not grab DBus proxy for ConsoleKit: %s", error->message);
-		g_error_free(error);
+		g_clear_error (&error);
 		return;
 	}
 
@@ -1257,6 +1218,7 @@ geo_address_cb (GeoclueAddress * address, int timestamp, GHashTable * addy_data,
 {
 	if (error != NULL) {
 		g_warning("Unable to get Geoclue address: %s", error->message);
+		g_clear_error (&error);
 		return;
 	}
 
@@ -1272,7 +1234,7 @@ geo_address_cb (GeoclueAddress * address, int timestamp, GHashTable * addy_data,
 		geo_timezone = g_strdup((gchar *)tz_hash);
 	}
 
-	check_timezone_sync();
+	update_location_menu_items();
 
 	return;
 }
@@ -1318,6 +1280,7 @@ geo_create_address (GeoclueMasterClient * master, GeoclueAddress * address, GErr
 {
 	if (error != NULL) {
 		g_warning("Unable to create GeoClue address: %s", error->message);
+		g_clear_error (&error);
 		return;
 	}
 
@@ -1344,6 +1307,7 @@ geo_req_set (GeoclueMasterClient * master, GError * error, gpointer user_data)
 {
 	if (error != NULL) {
 		g_warning("Unable to set Geoclue requirements: %s", error->message);
+		g_clear_error (&error);
 	}
 	return;
 }
@@ -1369,7 +1333,7 @@ geo_client_invalid (GeoclueMasterClient * client, gpointer user_data)
 		geo_timezone = NULL;
 	}
 
-	check_timezone_sync();
+	update_location_menu_items();
 
 	return;
 }
@@ -1391,7 +1355,7 @@ geo_address_change (GeoclueMasterClient * client, gchar * a, gchar * b, gchar *
 		geo_timezone = NULL;
 	}
 
-	check_timezone_sync();
+	update_location_menu_items();
 
 	return;
 }
@@ -1404,7 +1368,13 @@ geo_create_client (GeoclueMaster * master, GeoclueMasterClient * client, gchar *
 
 	geo_master = client;
 
-	if (geo_master != NULL) {
+	if (error != NULL) {
+		g_warning("Unable to get a GeoClue client!  '%s'  Geolocation based timezone support will not be available.", error->message);
+		g_clear_error (&error);
+		return;
+	}
+
+	if (geo_master == NULL) {
 		g_warning(_("Unable to get a GeoClue client!  Geolocation based timezone support will not be available."));
 		return;
 	}
@@ -1512,6 +1482,8 @@ main (int argc, char ** argv)
 	g_object_unref(G_OBJECT(server));
 	g_object_unref(G_OBJECT(root));
 
+	icaltimezone_free_builtin_timezones();
+
 	geo_address_clean();
 	geo_client_clean();
 
